import type { Control, FieldValues, Path } from 'react-hook-form';
import { type z } from 'zod';

import {
  FormControl,
  FormDescription,
  FormField,
  FormItem,
  FormLabel,
  FormMessage,
} from '~/components/ui/form';
import { RadioGroup, RadioGroupItem } from '~/components/ui/radio-group';
import { cn } from '~/lib/utils';

type EnumChoosingFieldProps<TFieldValues extends FieldValues> = {
  control: Control<TFieldValues> | undefined;
  name: Path<TFieldValues>;
  label: string;
  options: z.ZodEnum<[string, ...string[]]>;
  nextStep: () => Promise<void>;
};

const EnumChoosingField = <TFieldValues extends FieldValues>({
  control,
  name,
  label,
  options,
  nextStep,
}: EnumChoosingFieldProps<TFieldValues>) => {
  return (
    <FormField
      control={control}
      name={name}
      render={({ field }) => (
        <div className='w-10/12 md:w-1/3'>
          <FormLabel className='text-2xl font-bold'>{label}</FormLabel>
          <div className='flex w-full flex-col items-center justify-center'>
            <FormItem className='w-full'>
              <RadioGroup
                onValueChange={async (value) => {
                  field.onChange(value);
                  await nextStep();
                }}
                value={field.value}
                className={cn('mt-8 grid grid-cols-1 items-center gap-4')}
              >
                {options.options.map((item) => (
                  <FormItem key={item} className='mx-auto w-full max-w-3xl'>
                    <FormControl>
                      <RadioGroupItem
                        value={item}
                        id={item}
                        className='sr-only'
                        onClick={async () => {
                          if (field.value === item) {
                            await nextStep();
                          }
                        }}
                      />
                    </FormControl>
                    <FormLabel
                      htmlFor={item}
                      className={cn(
                        'relative flex w-full cursor-pointer items-center justify-center rounded-lg border-2 border-teal-600 px-6 py-4 text-center font-semibold transition-all duration-300 hover:border-teal-700 hover:bg-teal-700 hover:text-white sm:h-[120px]',
                        'shadow-md hover:shadow-lg',
                        field.value === item && 'border-teal-700 bg-teal-700 text-white shadow-lg'
                      )}
                    >
                      <div
                        className='text-lg leading-6 sm:text-xl'
                        style={{ fontWeight: 600, letterSpacing: 0.5 }}
                      >
                        <span>{item}</span>
                      </div>
                    </FormLabel>
                  </FormItem>
                ))}
              </RadioGroup>
            </FormItem>
            <FormMessage />
          </div>
          <FormDescription className='mt-2 text-start'>Scegli un&apos;opzione</FormDescription>
        </div>
      )}
    />
  );
};

export default EnumChoosingField;
